<!--

// Author: Tanmay K. Mohapatra

// Publisher Interface
// function getNumMessages(): returns integer specifying number of messages available
// function getMsgType(iMsgIndex): returns the message type (string) of message at position iMsgIndex
// function getMsg(iMsgIndex): returns the message (string) at position iMsgIndex
// function flushMsg(iMsgIndex): the message has been retrieved and can be deleted by the
//                               publisher.

// Subscriber Interface
// function acceptMsg(sMsgType, sMsg): accept a message with the corresponding message type

// Registration Procedure
// registerAsPublisher(classIn): Register the class classIn as a publisher. A publisher
//                               can publish messages of any type
// registerAsSubscriber(classIn, sMsgType): Register the class classIn as a subscriber 
//                                          for messages of type sMsgType. The same 
//                                          instance of a class can be registered multiple
//                                          times for the same of different message types.

// Starting and Stopping the Talkative Service
// function startTalk(): Starts interacting with publishers and subscribers at an 
//                       interval of iCheckInterval milliseconds.
// function stopTalk():  Stops interacting with publishers and subscriber.


var publisherArr = new Array();
var subscriberArr = new Array();
var subscriberMsgTypeArr = new Array();
var msgTypeArr = new Array();
var msgArr = new Array();
var bStarted = false;
var iCheckInterval = 1000; // check every second
var checkIntervalId;
var usedArr;

function registerAsPublisher(classIn)
{
    var iCurrLen = publisherArr.length;
    publisherArr[iCurrLen] = classIn;
}

function registerAsSubscriber(classIn, sMsgType)
{
    var iCurrLen = subscriberArr.length;
    subscriberArr[iCurrLen] = classIn;
    subscriberMsgTypeArr[iCurrLen] = sMsgType;
}

function checkMessages()
{
    // check all publishers. If message found, add to message queue.
    checkAndAddNewMessages();
    
    // check all messages. If subscriber found, feed subscriber.
    feedSubscribers();
    
    // delete fed messages
    deleteFedMessages();
}

function checkAndAddNewMessages()
{
    var iNumPublishers = publisherArr.length;
    var iIndex;
    var iNumTotalMessages = msgTypeArr.length;
    for(iIndex=0; iIndex < iNumPublishers; iIndex++)
    {
        var iNumMessages;
        var iMsgIndex;
        iNumMessages = publisherArr[iIndex].getNumMessages();
        for (iMsgIndex=0; iMsgIndex < iNumMessages; iMsgIndex++)
        {
            msgTypeArr[iNumTotalMessages] = publisherArr[iIndex].getMsgType(iMsgIndex);
            msgArr[iNumTotalMessages] = publisherArr[iIndex].getMsg(iMsgIndex);
            iNumTotalMessages++;
        }
        for (iMsgIndex=0; iMsgIndex < iNumMessages; iMsgIndex++)
        {
            publisherArr[iIndex].flushMsg(iMsgIndex);
        }
    }
}

function feedSubscribers()
{
    var iNumMessages = msgArr.length;
    var iNumSubscribers = subscriberArr.length;
    var iIndex;
    var iSubscriberIndex;

    usedArr = new Array();
    
    for(iIndex=0; iIndex < iNumMessages; iIndex++)
    {
        var bUsed = false;
        
        for(iSubscriberIndex=0; iSubscriberIndex < iNumSubscribers; iSubscriberIndex++)
        {
            // check if msg type is same as what subscriber wants
            if(msgTypeArr[iIndex] == subscriberMsgTypeArr[iSubscriberIndex])
            {
                subscriberArr[iSubscriberIndex].acceptMsg(msgTypeArr[iIndex], msgArr[iIndex]);
                bUsed = true;
            }
        }
        usedArr[iIndex] = bUsed;
    }
}

function deleteFedMessages()
{
    var newMsgArr = new Array();
    var newMsgTypeArr = new Array();
    var iNumMessages;
    var iOldIndex;
    var iNewIndex;
    
    iNumMessages = msgArr.length;
    for(iOldIndex=iNewIndex=0; iOldIndex < iNumMessages; iOldIndex++)
    {
        if(false == usedArr[iOldIndex])
        {
            newMsgArr[iNewIndex] = msgArr[iOldIndex];
            newMsgTypeArr[iNewIndex] = msgTypeArr[iOldIndex];
            iNewIndex++;
        }
    }
    msgArr = newMsgArr;
    msgTypeArr = newMsgTypeArr;
}

function startTalk()
{
    if (false == bStarted)
    {
        bStarted = true;
        checkIntervalId = setInterval("checkMessages()", iCheckInterval);
    }
}

function stopTalk()
{
    if (true == bStarted)
    {
        bStarted = false;
        clearInterval(checkIntervalId);
    }
}
//-->
